/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

import java.util.Enumeration;
import java.util.Vector;

class ComputerPlayer extends Player implements Runnable {
    private static final boolean DEBUG = false;
    private static final boolean FAST_FINISH = true;
    private static final int K = 128;

    private Thread thread = null;

    ComputerPlayer(String name, int player, GraphCanvas gc) {
	super(name, player, gc);
    }

    private boolean handlePath(Path path, int k) {
        int pathLength = path.size();

	if (FAST_FINISH && pathLength == 1) {
	    path.getEdge(0).value = 1f / 0;
	    return false;
	}

        int nodesSize = graph.nodes.size();
        //int edgesSize = graph.edges.size();

	for (Enumeration e = path.elements(); e.hasMoreElements(); ) {
	    //((Edge)e.nextElement()).value++;
            Edge edge = (Edge)e.nextElement();

            //int power = Math.min(edge.node1.multiNode.edges.size(),                                 
            //                     edge.node2.multiNode.edges.size());

// 	    edge.value += 
//                 (float)nodesSize / (path.size() + nodesSize - 1) + 
//                 (float)power / edgesSize;

            boolean narrowWay = 
                (edge.node1.multiNode != graph.start.multiNode &&
                 edge.node1.multiNode != graph.finish.multiNode &&
                 edge.node1.multiNode.edges.size() <= 2) ||
                (edge.node2.multiNode != graph.start.multiNode &&
                 edge.node2.multiNode != graph.finish.multiNode &&
                 edge.node2.multiNode.edges.size() <= 2);           

            float lc = 1f / pathLength;
 	    edge.value += lc * (narrowWay ? 0.75f : 1f);          
	}

        if (DEBUG) {
            System.err.println("* " + k + "; " + path.size());

            for (Enumeration e = path.elements(); e.hasMoreElements();) {
                ((Edge) e.nextElement()).marked = true;
            }

            canvas.repaint();

            try {
                Thread.sleep(100);
            } catch (Exception e) {
            }
            
            for (Enumeration e = path.elements(); e.hasMoreElements();) {
                ((Edge) e.nextElement()).marked = false;
            }

            canvas.repaint();
	}

	return true;
    }

    private final Vector paths = new Vector();

    private void ien(MultiNode v1, MultiNode v2, int k) {	
	paths.removeAllElements();

	Path p1 = graph.shortestPath(v1, v2);  // path[t-1]
	Path p2;

	if (p1 == null) return;

	paths.addElement(p1);
	p1.marked = true;
	if (!handlePath(p1, 0)) return;

	for (int t = 1; t < k; t++) {
	    // 3 - 6
	    for (int i = 0; i < p1.size(); i++) {
		// 3
		for (Enumeration e = paths.elements(); e.hasMoreElements(); ) {
		    p2 = (Path)e.nextElement();
		    boolean b = true;
		    for (int ii = 0; ii < i; ii++) {
			if (p1.getEdge(ii) != p2.getEdge(ii)) {
			    
			    b = false;
			    break;
			}			
		    }
		    if (b) p2.getEdge(i).enabled = false;
		}

		// 4
		MultiNode removed = graph.start.multiNode;
		for (int ii = 0; ii < i; ii++) {
		    // temporary remove node
		    // TODO: optimize
		    for (Enumeration e = removed.edges.elements(); 
                         e.hasMoreElements(); ) {

			((Edge)e.nextElement()).enabled = false;
		    }
		    removed = p1.getEdge(ii).getAnotherNode(removed);
		}

		p2 = graph.shortestPath(removed, v2);

		if (p2 != null) {
		    // 5
		    for (int ii = i - 1; ii >= 0; ii--) {
 			p2.addElement(p1.getEdge(ii));
 		    }
		    paths.addElement(p2);
		}
		
		// 6
		for (Enumeration e = graph.edges.elements(); e.hasMoreElements(); ) {
		    ((Edge)e.nextElement()).enabled = true;
		}
	    }

	    // 7
	    p1 = null;
	    for (int i = 0; i < paths.size(); i++) {
		Path next = (Path)paths.elementAt(i);		
		if (!next.marked) {
		    if (p1 == null || next.size() < p1.size()) {
			p1 = next;
		    }
		}
	    }
	    
	    if (p1 == null) break;

	    p1.marked = true;
	    if (!handlePath(p1, t)) break;
	}
    }

    private Edge findEdge() {
        if (DEBUG) {
            try {
                Thread.sleep(1000);
            } catch (Exception e) {
            }
            
            System.err.println();
        }

	for (Enumeration e = graph.edges.elements(); e.hasMoreElements(); ) {
	    ((Edge)e.nextElement()).value = 0;
	}

	ien(graph.start.multiNode, graph.finish.multiNode, K);

	Edge bestEdge = null;

	for (Enumeration e = graph.edges.elements(); e.hasMoreElements(); ) {
	    Edge edge = (Edge)e.nextElement();
	    edge.enabled = true;
	    if (bestEdge == null || edge.value > bestEdge.value) {
		bestEdge = edge;
	    }
	}

	return bestEdge;
    }

    protected void move() {
	thread = new Thread(this);
	thread.start();
    }

    public void run() {
	long t = System.currentTimeMillis();
	System.out.print(this + ": Thinking...");

	Edge edgeToDo = findEdge();

	System.out.println
            ("OK (" + (System.currentTimeMillis() - t) / 1000D + " s)");

	synchronized (this) {
	    if (Thread.currentThread() != thread)
		return;

	    switch (player) {
	    case CUT:
		graph.removeEdge(edgeToDo);
		break;
	    case SHORT:
		edgeToDo.shorten();
	    }

	    done();
	}

	canvas.repaint(edgeToDo);
    }

    public synchronized void stop() {
	thread = null;
	super.stop();
    }
}
