/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

import java.awt.Point;
import java.util.Enumeration;
import java.util.Vector;

class Node extends Point {
    Vector edges = new Vector();
    Vector connectedEdges = new Vector();
    Graph graph = null;
    MultiNode multiNode;

    Node(int x, int y, Graph graph) {
	super(x, y);
	this.graph = graph;
	this.multiNode = new MultiNode(graph);
	multiNode.nodes.addElement(this);
    }

    Node(int x, int y) {
	this(x, y, null);
    }

    public boolean equals(Object o) {
	if (o instanceof Node) {
	    Node gn = (Node)o;
	    if (this.x == gn.x & this.y == gn.y)
		return true;
	}
	return false;
    }

    public int distTo(Node node) {
	int nx2 = node.x - x; nx2 = nx2 * nx2;
	int ny2 = node.y - y; ny2 = ny2 * ny2;
	return Math.round((float)Math.sqrt(nx2 + ny2));
    }

    public void addLinkedNodes(Vector nodes) {
	nodes.addElement(this);
	Enumeration iter = edges.elements();
	while (iter.hasMoreElements()) {
	    Edge link = (Edge)iter.nextElement();
            nodes.addElement(link.getAnotherNode(this));
	}
    }
    
    public void addConnectedNodes(Vector nodes) {
        nodes.addElement(this);
        Enumeration iter = connectedEdges.elements();
        while (iter.hasMoreElements()) {
            Edge link = (Edge) iter.nextElement();
            Node node = link.getAnotherNode(this);
            if (!nodes.contains(node)) {
                node.addConnectedNodes(nodes);
            }
        }
    }
    
    public void addConnectedLinkedNodes(Vector nodes) {
	Vector v = new Vector();
	addConnectedNodes(v);
	Enumeration enu = v.elements();
	while(enu.hasMoreElements()) {
	    Node node = (Node)enu.nextElement();
	    node.addLinkedNodes(nodes);
	}
	nodes.removeElement(this);
    }
}
